"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _extend = _interopRequireDefault(require("./extend"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var Node = /*#__PURE__*/function () {
  function Node(node) {
    _classCallCheck(this, Node);

    _defineProperty(this, "id", null);

    _defineProperty(this, "parent", null);

    _defineProperty(this, "children", []);

    _defineProperty(this, "state", {});

    (0, _extend["default"])(this, node);
    this.children = this.children || [];
  } // Returns a boolean value indicating whether a node is a descendant of a given node or not.
  // @param {object} node Specifies the node that may be contained by (a descendant of) a specified node.
  // @return {boolean} Returns true if a node is a descendant of a specified node, otherwise false. A descendant can be a child, grandchild, great-grandchild, and so on.


  _createClass(Node, [{
    key: "contains",
    value: function contains(node) {
      while (node instanceof Node && node !== this) {
        if (node.parent === this) {
          return true;
        }

        node = node.parent;
      }

      return false;
    } // Gets a child node at the specified index.
    // @param {number} The index of the child node.
    // @return {object} Returns an object that defines the node, null otherwise.

  }, {
    key: "getChildAt",
    value: function getChildAt(index) {
      var node = null;

      if (this.children.length > 0 && index >= 0 && index < this.children.length) {
        node = this.children[index];
      }

      return node;
    } // Gets the child nodes.
    // @return {array} Returns an array of objects that define the nodes.

  }, {
    key: "getChildren",
    value: function getChildren() {
      return this.children;
    } // Gets the first child node.
    // @return {object} Returns an object that defines the node, null otherwise.

  }, {
    key: "getFirstChild",
    value: function getFirstChild() {
      var node = null;

      if (this.children.length > 0) {
        node = this.children[0];
      }

      return node;
    } // Gets the last child node.
    // @return {object} Returns an object that defines the node, null otherwise.

  }, {
    key: "getLastChild",
    value: function getLastChild() {
      var node = null;

      if (this.children.length > 0) {
        node = this.children[this.children.length - 1];
      }

      return node;
    } // Gets the next sibling node.
    // @return {object} Returns an object that defines the node, null otherwise.

  }, {
    key: "getNextSibling",
    value: function getNextSibling() {
      var node = null;

      if (this.parent) {
        var index = this.parent.children.indexOf(this);

        if (index >= 0 && index < this.parent.children.length - 1) {
          node = this.parent.children[index + 1];
        }
      }

      return node;
    } // Gets the parent node.
    // @return {object} Returns an object that defines the node, null otherwise.

  }, {
    key: "getParent",
    value: function getParent() {
      return this.parent;
    } // Gets the previous sibling node.
    // @return {object} Returns an object that defines the node, null otherwise.

  }, {
    key: "getPreviousSibling",
    value: function getPreviousSibling() {
      var node = null;

      if (this.parent) {
        var index = this.parent.children.indexOf(this);

        if (index > 0 && index < this.parent.children.length) {
          node = this.parent.children[index - 1];
        }
      }

      return node;
    } // Checks whether this node has children.
    // @return {boolean} Returns true if the node has children, false otherwise.

  }, {
    key: "hasChildren",
    value: function hasChildren() {
      return this.children.length > 0;
    } // Checks whether this node is the last child of its parent.
    // @return {boolean} Returns true if the node is the last child of its parent, false otherwise.

  }, {
    key: "isLastChild",
    value: function isLastChild() {
      var hasNextSibling = this.getNextSibling();
      return !hasNextSibling;
    }
  }]);

  return Node;
}();

var _default = Node;
exports["default"] = _default;